#ifndef MHACHE_COLOR_H
#define MHACHE_COLOR_H

#include <unordered_map>
#include <string>
#include <array>

class MHacheColor {
private:
    std::array<unsigned char, 4> _rgba;

private:
    MHacheColor() = delete;

    static constexpr unsigned char touccolor(float value)
    {
        return static_cast<unsigned char>(value * 255.0f);
    };

    static constexpr float tofcolor(unsigned char value)
    {
        constexpr const float conversion = 1/255.f;
        return static_cast<float>(value) * conversion;
    };

    static std::unordered_map<std::string, MHacheColor> _colors;

public:
    constexpr MHacheColor(unsigned char r, unsigned char g, unsigned char b, unsigned char a = 255u);
    constexpr MHacheColor(int r, int g, int b, int a = 255);
    constexpr MHacheColor(float r, float g, float b, float a = 1.0f);

    MHacheColor(const MHacheColor&)                 = default;
    MHacheColor(MHacheColor&&)                      = default;
    MHacheColor& operator = (const MHacheColor&)    = default;
    MHacheColor& operator = (MHacheColor&&)         = default;
    ~MHacheColor()                                  = default;

    template<typename T> T red()    const;
    template<typename T> T green()  const;
    template<typename T> T blue()   const;
    template<typename T> T alpha()  const;

    template <typename T> std::array<T, 4> channels() const;

    static MHacheColor light_blue();
    static MHacheColor blue();
    static MHacheColor dark_blue();

    static MHacheColor light_orange();
    static MHacheColor orange();
    static MHacheColor dark_orange();

    static MHacheColor green();
    static MHacheColor pink();
    static MHacheColor purple();
    static MHacheColor gray();
    static MHacheColor white();
    static MHacheColor yellow();

    static MHacheColor* get_color(const std::string&);
    static MHacheColor* save_color(const std::string&, MHacheColor&&);
};

#endif
