#ifndef MHACHE_SKILL_H
#define MHACHE_SKILL_H

#include <string>
#include <vector>

#include "mhache_color.h"

class MHacheSkill {
private:
    std::string                 _name;
    std::string                 _description;
    const MHacheColor*          _color;
    std::vector<std::string>    _levels;

private:
    MHacheSkill()                           = delete;
    MHacheSkill(const MHacheSkill&)         = delete;
    MHacheSkill(MHacheSkill&&)              = delete;
    MHacheSkill& operator=(MHacheSkill&)    = delete;
    MHacheSkill& operator=(MHacheSkill&&)   = delete;

public:
    inline 
    MHacheSkill(std::string_view name, std::string_view description, const MHacheColor* color, std::vector<std::string>&& levels)
        : _name(name), _description(description), _color(color), _levels(std::move(levels)) 
    {}

    ~MHacheSkill() = default;
    
    inline const std::string& name() const                  { return _name; }
    inline const std::string& description() const           { return _description; }
    inline const std::vector<std::string>& levels() const   { return _levels; }
    inline const MHacheColor* color() const                 { return _color; }

    inline bool operator <  (const MHacheSkill& obj) const { return _name  < obj._name; };
    inline bool operator == (const MHacheSkill& obj) const { return _name == obj._name; };
};

template<>
struct std::hash<MHacheSkill> 
{
    size_t operator() (const MHacheSkill& skill) const
    {
        return std::hash<std::string>{}(skill.name());
    }
};

struct MHacheSkillLessOperator {
    using is_transparent = void;

    bool operator() (const MHacheSkill& a, const MHacheSkill& b) const { return a < b; }
    bool operator() (const MHacheSkill& a, std::string_view b) const { return a.name() < b; }
    bool operator() (std::string_view a, const MHacheSkill& b) const { return a < b.name(); }
};

#endif
