#ifndef MHACHE_WINDOW_H
#define MHACHE_WINDOW_H

#include <csignal>
#include <GLFW/glfw3.h>

class MHacheWindow {
private:
    int _width;
    int _height;

    GLFWwindow* _window;

    static constexpr const int _opengl_major_version = 3;
    static constexpr const int _opengl_minor_version = 0;

    static constexpr const char* _title         = "MHache 0.1";
    static constexpr const char* _GLSL_version  = "#version 130";

    // Handle external signals:
    static volatile std::sig_atomic_t _signal_status;

private:
    static void callback_signals(int signal);
    static void callback_glfw_error(int error, const char* description);
    static void callback_glfw_keyboard(GLFWwindow* window, int key, int scancode, int action, int mods);

    void new_frame();
    void render_frame();

    void build_widgets();
    void build_menu();

    MHacheWindow()                      = delete;
    MHacheWindow(const MHacheWindow&)   = delete;
    MHacheWindow(MHacheWindow&&)        = delete;

public:
    MHacheWindow(int width, int height);
    ~MHacheWindow();

    void resize(int width, int height);
    bool ready();
    void display();
};

#endif
