#include "mhache_color.h"
#include <unordered_map>

constexpr MHacheColor::MHacheColor(unsigned char r, unsigned char g, unsigned char b, unsigned char a)
    : _rgba({ r, g, b, a })
{}

constexpr MHacheColor::MHacheColor(int r, int g, int b, int a /* = 255 */)
    : MHacheColor(
        static_cast<unsigned char>(r),
        static_cast<unsigned char>(g),
        static_cast<unsigned char>(b),
        static_cast<unsigned char>(a))
{}

constexpr MHacheColor::MHacheColor(float r, float g, float b, float a)
    : MHacheColor(touccolor(r), touccolor(g), touccolor(b), touccolor(a))
{}

template<> unsigned char MHacheColor::red<unsigned char>() const    { return _rgba[0]; }
template<> unsigned char MHacheColor::green<unsigned char>() const  { return _rgba[1]; }
template<> unsigned char MHacheColor::blue<unsigned char>() const   { return _rgba[2]; }
template<> unsigned char MHacheColor::alpha<unsigned char>() const  { return _rgba[3]; }

template<> std::array<unsigned char, 4> MHacheColor::channels() const
{    
    return _rgba; 
}

template<> float MHacheColor::red<float>() const    { return tofcolor(_rgba[0]); }
template<> float MHacheColor::green<float>() const  { return tofcolor(_rgba[1]); }
template<> float MHacheColor::blue<float>() const   { return tofcolor(_rgba[2]); }
template<> float MHacheColor::alpha<float>() const  { return tofcolor(_rgba[3]); }

template<> std::array<float, 4> MHacheColor::channels() const
{    
    return { red<float>(), green<float>(), blue<float>(), alpha<float>() };
}

MHacheColor MHacheColor::light_blue()   { return MHacheColor(164,   204,    254);   }
MHacheColor MHacheColor::blue()         { return MHacheColor(96,    169,    250);   }
MHacheColor MHacheColor::dark_blue()    { return MHacheColor(77,    105,    209);   }

MHacheColor MHacheColor::light_orange() { return MHacheColor(236,   167,    82);    }
MHacheColor MHacheColor::orange()       { return MHacheColor(225,   99,     18);    }
MHacheColor MHacheColor::dark_orange()  { return MHacheColor(248,   77,     20);    }

MHacheColor MHacheColor::green()        { return MHacheColor(39,    222,    150);   }
MHacheColor MHacheColor::pink()         { return MHacheColor(227,   116,    179);   }
MHacheColor MHacheColor::purple()       { return MHacheColor(158,   62,     212);   }
MHacheColor MHacheColor::gray()         { return MHacheColor(169,   168,    168);   }
MHacheColor MHacheColor::white()        { return MHacheColor(252,   251,    251);   }
MHacheColor MHacheColor::yellow()       { return MHacheColor(249,   250,    116);   }

std::unordered_map<std::string, MHacheColor> MHacheColor::_colors = 
{
    { "light blue",     light_blue()    },
    { "blue",           blue()          },
    { "dark blue",      dark_blue()     },
    { "light orange",   light_orange()  },
    { "orange",         orange()        },
    { "dark orange",    dark_orange()   },
    { "green",          green()         },
    { "pink",           pink()          },
    { "purple",         purple()        },
    { "gray",           gray()          },
    { "white",          white()         },
    { "yellow",         yellow()        },
};

MHacheColor*
MHacheColor::get_color(const std::string& name)
{
    const auto it = _colors.find(name);
    return (_colors.end() == it) ? nullptr : &(it->second);
}

MHacheColor*
MHacheColor::save_color(const std::string& name, MHacheColor&& color)
{
    return &(_colors.insert_or_assign(name, std::forward<MHacheColor>(color)).first->second);
}
